@@ -15,9 +15,10 @@ from contextlib import closing
 from itertools import chain
 import json
 import sys
-
+import requests
+import urllib.request, urllib.parse, urllib.error
 import yaml
-from bottle import get, post, local, request
+from bottle import get, post, local, request, abort, httplib
 
 from codalab.common import UsageError, NotFoundError
 from codalab.lib import formatting, spec_util
@@ -39,7 +40,12 @@ from codalab.model.tables import GROUP_OBJECT_PERMISSION_ALL
 from codalab.objects.permission import permission_str
 from codalab.rest import util as rest_util
 from codalab.rest.worksheets import get_worksheet_info, search_worksheets
-from codalab.rest.worksheet_block_schemas import BlockModes, MarkupBlockSchema, FetchStatusCodes
+from codalab.rest.worksheet_block_schemas import (
+    BlockModes,
+    MarkupBlockSchema,
+    FetchStatusCodes,
+    FetchStatusSchema,
+)
 from codalab.worker.download_util import BundleTarget
 
 
@@ -66,7 +72,7 @@ def _interpret_search():
 @post('/interpret/wsearch')
 def _interpret_wsearch():
     """
-    Returns worksheet items given a search query for worksheets.
+    Returns worksheets information given a search query for worksheets.
 
     JSON request body:
     ```
@@ -74,8 +80,34 @@ def _interpret_wsearch():
         "keywords": [ list of search keywords ]
     }
     ```
+
+    Response body:
+    ```
+    {
+        "response": [
+            {id: 6,
+            uuid: "0x5505f540936f4d0d919f3186141192b0",
+            name: "codalab-a",
+            title: "CodaLab Dashboard",
+            frozen: null,
+            owner_id: "0"
+            owner_name: "codalab"
+            group_permissions: {
+                id: 8,
+                group_uuid: "0x41e95d8592de417cbb726085d6986137",
+                group_name: "public",
+                permission: 1}
+            }
+            ...
+        ]
+    }
+    ```
     """
-    return interpret_wsearch(request.json)
+    query = request.json
+    if 'keywords' not in query:
+        abort(httplib.BAD_REQUEST, 'Missing `keywords`')
+
+    return {'response': interpret_wsearch(query['keywords'])}
 
 
 @post('/interpret/file-genpaths')
@@ -158,6 +190,35 @@ def fetch_interpreted_worksheet(uuid):
     - resolve_interpreted_items: get more information about a worksheet.
     In the future, for large worksheets, might want to break this up so
     that we can render something basic.
+    Return: 
+        worksheet_info dict{}:
+            key:[value_type] <description>
+            blocks:[list] 
+                    Resolved worksheet blocks from raw_items.
+                        Bundles will be grouped into table block items, 
+                        text items might be grouped into one markdown block etc.
+            source:[list] source lines
+            raw_to_block:[list] 
+                            Raw_items to its block index pair.
+                                For example, assume the first resolved block item is a bundle table that has 2 rows,
+                                then the 2nd element in the list would be [0, 1]
+                                [0, 1]: 0 means the item belongs to the first block,
+                                        1 means the item is the second item of the block (2nd bundle in our example)
+                                NOTE: Used for setting focus on frontend
+            block_to_raw:[dict] 
+                            Maps the blocks (table, markdown, records) to their corresponding source line indices,
+                            it's mostly a reverse mapping of raw_to_block, by mostly: raw_to_block has some bug,
+                            please refer to worksheet_utils flush_bundles function.
+                            This can be used to index the source on the frontend
+                            Example:
+                            [0, 0]: 0
+                            [0, 1]: 1
+                            [1, 0]: 9
+                            This means the first blocks' first item corresponds to the first line in source,
+                            the second item corresponds to the second line in source
+                            The second block corresponds the 10th line in source. 
+                            2-8 can be skipped for multiple reasons: blank lines, comments, schema lines etc.
+                                NOTE: Used for setting focus on frontend                                      
 
     This endpoint can be called with &brief=1 in order to give an abbreviated version,
     which does not resolve searches or wsearches.
@@ -170,13 +231,13 @@ def fetch_interpreted_worksheet(uuid):
     brief = request.query.get("brief", "0") == "1"
 
     directive = request.query.get("directive", None)
+    print(directive)
     search_results = []
 
     worksheet_info = get_worksheet_info(uuid, fetch_items=True, fetch_permissions=True)
 
     # Shim in additional data for the frontend
     worksheet_info['items'] = resolve_items_into_infos(worksheet_info['items'])
-    worksheet_info['raw'] = get_worksheet_lines(worksheet_info)
 
     if worksheet_info['owner_id'] is None:
         worksheet_info['owner_name'] = None
@@ -184,11 +245,17 @@ def fetch_interpreted_worksheet(uuid):
         owner = local.model.get_user(user_id=worksheet_info['owner_id'])
         worksheet_info['owner_name'] = owner.user_name
 
-    # Replace searches with raw items.
-    # This needs to be done before get_worksheet_lines because this replaces
-    # user-written raw items.
+    # Fetch items.
+    worksheet_info['source'] = get_worksheet_lines(worksheet_info)
+
     if not directive and not brief:
-        worksheet_info['items'] = expand_search_items(worksheet_info['items'])
+        expanded_items = []
+        for index, raw_item in enumerate(worksheet_info['items']):
+            expanded = expand_search_item(raw_item)
+            expanded_items.append(expanded)
+            # Multiple items can correspond to the same source line (i.e: search directives)
+            # raw_items_to_source_index.extend([index] * len(expanded))
+        worksheet_info['items'] = list(chain.from_iterable(expanded_items))
     elif directive:
         # Only expand the search item corresponding to the given directive.
         # Used in async loading to only load a single table.
@@ -243,30 +310,30 @@ def fetch_interpreted_worksheet(uuid):
                         block['bundle_info'][j] = None
                 if not is_relevant_block:
                     interpreted_blocks['blocks'][i] = None
-
-    worksheet_info['items'] = resolve_interpreted_blocks(interpreted_blocks['blocks'])
+    # Grouped individual items into blocks
+    worksheet_info['blocks'] = resolve_interpreted_blocks(interpreted_blocks['blocks'], brief=brief)
     worksheet_info['raw_to_block'] = interpreted_blocks['raw_to_block']
     worksheet_info['block_to_raw'] = interpreted_blocks['block_to_raw']
 
     if directive:
         # If we're only async loading a single table_block / subworksheets_block,
         # return only that block (which is at the end of worksheet_info['items'])
-        worksheet_info['items'] = [worksheet_info['items'][-1]] if len(search_results) else []
+        worksheet_info['blocks'] = [worksheet_info['blocks'][-1]] if len(search_results) else []
 
-    for item in worksheet_info['items']:
-        if item is None:
+    for block in worksheet_info['blocks']:
+        if block is None:
             continue
-        if item['mode'] == 'table':
-            for row_map in item['rows']:
+        if block['mode'] == 'table':
+            for row_map in block['rows']:
                 for k, v in row_map.items():
                     if v is None:
                         row_map[k] = formatting.contents_str(v)
-        if 'bundle_info' in item:
+        if 'bundle_info' in block:
             infos = []
-            if isinstance(item['bundle_info'], list):
-                infos = item['bundle_info']
-            elif isinstance(item['bundle_info'], dict):
-                infos = [item['bundle_info']]
+            if isinstance(block['bundle_info'], list):
+                infos = block['bundle_info']
+            elif isinstance(block['bundle_info'], dict):
+                infos = [block['bundle_info']]
             for bundle_info in infos:
                 if bundle_info is None:
                     continue
@@ -274,8 +341,10 @@ def fetch_interpreted_worksheet(uuid):
                     continue  # empty info: invalid bundle reference
                 if isinstance(bundle_info, dict):
                     format_metadata(bundle_info.get('metadata'))
+    # Frontend doesn't use individual 'items' for now
+    del worksheet_info['items']
     if bundle_uuids:
-        return {'items': worksheet_info['items']}
+        return {'blocks': worksheet_info['blocks']}
     return worksheet_info
 
 
@@ -324,7 +393,7 @@ def head_target(target, max_num_lines):
 DEFAULT_GRAPH_MAX_LINES = 100
 
 
-def resolve_interpreted_blocks(interpreted_blocks):
+def resolve_interpreted_blocks(interpreted_blocks, brief):
     """
     Called by the web interface.  Takes a list of interpreted worksheet
     items (returned by worksheet_util.interpret_items) and fetches the
@@ -349,11 +418,20 @@ def resolve_interpreted_blocks(interpreted_blocks):
                 pass
             elif mode == BlockModes.record_block or mode == BlockModes.table_block:
                 # header_name_posts is a list of (name, post-processing) pairs.
-                contents = block['rows']
                 # Request information
-                contents = interpret_genpath_table_contents(contents)
-
-                block['rows'] = contents
+                if brief:
+                    # In brief mode, only calculate whether we should interpret genpaths, and if so, set status to briefly_loaded.
+                    should_interpret_genpaths = (
+                        len(get_genpaths_table_contents_requests(block['rows'])) > 0
+                    )
+                    block['status'] = (
+                        FetchStatusSchema.get_briefly_loaded_status()
+                        if should_interpret_genpaths
+                        else FetchStatusSchema.get_ready_status()
+                    )
+                else:
+                    block['rows'] = interpret_genpath_table_contents(block['rows'])
+                    block['status'] = FetchStatusSchema.get_ready_status()
             elif mode == BlockModes.contents_block or mode == BlockModes.image_block:
                 bundle_uuid = block['bundles_spec']['bundle_infos'][0]['uuid']
                 target_path = block['target_genpath']
@@ -423,6 +501,36 @@ def resolve_interpreted_blocks(interpreted_blocks):
     return interpreted_blocks
 
 
+def interpret_wsearch(keywords):
+    """
+    Return a list of row dicts, one per worksheet. These dicts do NOT contain
+    ALL worksheet items; this method is meant to make it easy for a user to see
+    their existing worksheets.
+
+    Each keyword is either:
+    - <key>=<value>
+    - .floating: return bundles not in any worksheet
+    - .offset=<int>: return bundles starting at this offset
+    - .limit=<int>: maximum number of bundles to return
+    - .count: just return the number of bundles
+    - .shared: shared with me through a group
+    - .mine: sugar for owner_id=user_id
+    - .last: sugar for id=.sort-
+    Keys are one of the following:
+    - Bundle fields (e.g., uuid)
+    - Metadata fields (e.g., time)
+    - Special fields (e.g., dependencies)
+    Values can be one of the following:
+    - .sort: sort in increasing order
+    - .sort-: sort by decreasing order
+    - .sum: add up the numbers
+    Bare keywords: sugar for uuid_name=.*<word>.*
+    Search only bundles which are readable by user_id.
+    """
+
+    return search_worksheets(keywords)
+
+
 def is_bundle_genpath_triple(value):
     # if called after an RPC call tuples may become lists
     need_gen_types = (tuple, list)
@@ -430,6 +538,24 @@ def is_bundle_genpath_triple(value):
     return isinstance(value, need_gen_types) and len(value) == 3
 
 
+def get_genpaths_table_contents_requests(contents):
+    """
+    Get genpath requests to fill in values for a table.
+
+    contents represents a table, but some of the elements might not be
+    interpreted yet, so fill them in.
+    
+    Returns requests: list of (bundle_uuid, genpath, post-processing-func)
+    """
+    requests = []
+    for r, row in enumerate(contents):
+        for key, value in row.items():
+            # value can be either a string (already rendered) or a (bundle_uuid, genpath, post) triple
+            if is_bundle_genpath_triple(value):
+                requests.append(value)
+    return requests
+
+
 def interpret_genpath_table_contents(contents):
     """
     contents represents a table, but some of the elements might not be
@@ -437,12 +563,7 @@ def interpret_genpath_table_contents(contents):
     """
 
     # Request information
-    requests = []
-    for r, row in enumerate(contents):
-        for key, value in row.items():
-            # value can be either a string (already rendered) or a (bundle_uuid, genpath, post) triple
-            if is_bundle_genpath_triple(value):
-                requests.append(value)
+    requests = get_genpaths_table_contents_requests(contents)
     responses = interpret_file_genpaths(requests)
 
     # Put it in a table
@@ -571,8 +692,7 @@ def resolve_items_into_infos(items):
             else None
         )
         if bundle_info is not None:
-            bundle_info['sort_key'] = i['sort_key']
-            bundle_info['id'] = i['id']
+            bundle_info = dict(bundle_info, id=i['id'], sort_key=i['sort_key'])
         if i['subworksheet_uuid']:
             try:
                 subworksheet_info = local.model.get_worksheet(
@@ -626,17 +746,12 @@ def perform_search_query(value_obj):
             worksheet_infos = search_worksheets(keywords)
             for worksheet_info in worksheet_infos:
                 raw_items.append(subworksheet_item(worksheet_info) + (None, None))
-
         return raw_items
     else:
         # Not a search query
         return []
 
 
-def expand_search_items(raw_items):
-    return list(chain.from_iterable([expand_search_item(raw_item) for raw_item in raw_items]))
-
-
 def expand_search_item(raw_item):
     """
     Raw items that include searches must be expanded into more raw items.
