@@ -33,13 +33,18 @@ class BlockModes:
 
 
 class FetchStatusCodes:
+    """
+    The values here correspond with FETCH_STATUS_SCHEMA in the frontend.
+    """
+
     unknown = 'unknown'
     pending = 'pending'
+    briefly_loaded = 'briefly_loaded'
     ready = 'ready'
     not_found = 'not_found'
     no_permission = 'no_permission'
 
-    values = (unknown, pending, ready, not_found, no_permission)
+    values = (unknown, pending, briefly_loaded, ready, not_found, no_permission)
 
 
 class FetchStatusSchema(PlainSchema):
@@ -58,6 +63,10 @@ class FetchStatusSchema(PlainSchema):
     def get_pending_status():
         return {'code': FetchStatusCodes.pending, 'error_message': ''}
 
+    @staticmethod
+    def get_briefly_loaded_status():
+        return {'code': FetchStatusCodes.briefly_loaded, 'error_message': ''}
+
     @staticmethod
     def get_ready_status():
         return {'code': FetchStatusCodes.ready, 'error_message': ''}
@@ -147,6 +156,7 @@ class TableBlockSchema(WorksheetBlockSchema):
 
     header = fields.List(fields.String(), required=True)
     rows = fields.List(fields.Dict(), required=True)
+    sort_keys = fields.List(fields.Integer())
 
 
 class RecordsRowSchema(PlainSchema):
@@ -161,6 +171,7 @@ class RecordsBlockSchema(BundleBlockSchema):
 
     header = fields.Constant(('key', 'value'))
     rows = fields.Nested(RecordsRowSchema, many=True, required=True)
+    sort_keys = fields.List(fields.Integer())
 
 
 class GraphTrajectorySchema(PlainSchema):
@@ -184,8 +195,10 @@ class GraphBlockSchema(BundleBlockSchema):
 class SubworksheetsBlock(WorksheetBlockSchema):
     mode = fields.Constant(BlockModes.subworksheets_block)
     subworksheet_infos = fields.List(fields.Dict, required=True)
+    sort_keys = fields.List(fields.Integer())
 
 
 class PlaceholderBlockSchema(WorksheetBlockSchema):
     mode = fields.Constant(BlockModes.placeholder_block)
     directive = fields.String()
+    sort_keys = fields.List(fields.Integer())
